/*****************************************************************************/
/*                                                                           */
/* UNIT: NTL1_Error_Record  (Level 1 library routine)                        */
/*                                                                           */
/* Author: Nikola Stojanovic                                                 */
/*                                                                           */
/* Revision:    07 JUL 94   Version 1.0                                      */
/*                                                                           */
/* Function:                                                                 */
/*                                                                           */
/*   Procedure receives intended contents for the error buffer, allocates    */
/* the buffer and fills in its contents (error comment is copied to newly    */
/* allocated space, so string containing it should be released or re-used    */
/* by the caller); returns the assembled error buffer                        */
/*                                                                           */
/*****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ntl1.h"


/*****************************************************************************/

errind NTL1_Error_Record (int severity, int code,
                          char *comment, int description)
{
 errind new_record; char *augmented_report;

 new_record = (errind) NTL0_ckalloc (sizeof (Error_Struct));
 new_record -> kind = severity; new_record -> code = code;

 switch (severity) {
  case FATAL_ERROR: {
   if (comment == NULL) augmented_report = NULL;
   else {
    augmented_report = (char *) NTL0_ckalloc (
                                (strlen (comment) + strlen ("FATAL: ") + 2) *
                                                                sizeof (char));
    sprintf (augmented_report, "FATAL: %s.", comment);
   }
   break;
  }
  case USER_ERROR: {
   if (comment == NULL) augmented_report = NULL;
   else {
    augmented_report = (char *) NTL0_ckalloc (
                                (strlen (comment) + strlen ("ERROR: ") + 2) *
                                                                sizeof (char));
    sprintf (augmented_report, "ERROR: %s.", comment);
   }
   break;
  }
  case WARNING: {
   if (comment == NULL) augmented_report = NULL;
   else {
    augmented_report = (char *) NTL0_ckalloc (
                              (strlen (comment) + strlen ("WARNING: ") + 2) *
                                                                sizeof (char));
    sprintf (augmented_report, "WARNING: %s.", comment);
   }
   break;
  }
  default: {
   if (comment == NULL) augmented_report = NULL;
   else {
    augmented_report = (char *) NTL0_ckalloc (
                      (strlen (comment) + strlen ("UNKNOWN PROBLEM: ") + 2) *
                                                                sizeof (char));
    sprintf (augmented_report, "UNKNOWN PROBLEM: %s.", comment);
   }
  }
 }
 if (augmented_report == NULL) new_record -> message = NULL;
 else new_record -> message = augmented_report;

 new_record -> description = description;

 return new_record;
}

