/*****************************************************************************/
/*                                                                           */
/* UNIT: NTL3_Load_Alignment_1  (Level 3 library routine)                    */
/*                                                                           */
/* Author: Nikola Stojanovic                                                 */
/*                                                                           */
/* Revision:    17 JUL 94   Version 1.0                                      */
/*                                                                           */
/* Function:                                                                 */
/*                                                                           */
/*   Procedure loads the first alignment within the alignment file with      */
/* specified name into internal structure and returns the pointer to the     */
/* structure; returns the error status (record), NULL if there were no       */
/* errors                                                                    */
/*                                                                           */
/*****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include "ntl3.h"


/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* PROCEDURE: NTL3_Load_Alignment_1                                          */
/*                                                                           */
/* Central procedure for the unit; returns NULL if alignment is correctly    */
/*   loaded, allocated error record otherwise                                */

errind NTL3_Load_Alignment_1 (char *file_name, align_ptr *alignment)
{
 header_ptr file_data; errind load_status;

 /* Load the contents of the specified alignment file in appropriate format  */
 /*   into internal structure first                                          */

 if ((load_status = NTL2_Load_AlignFile (file_name, &file_data)) != NULL)
  return load_status;
 else {       /* Data from the file loaded, now partially expand alignment 1 */
  load_status = NTL1_Load_Alignment (1, file_data, alignment);
  file_data = NTL1_Destroy_Header (file_data);       /* File data not needed */
  return load_status;
 }
}

