/*****************************************************************************/
/*                                                                           */
/* UNIT: overlaps  (Utility interface level library routine)                 */
/*                                                                           */
/* Author: Nikola Stojanovic                                                 */
/*                                                                           */
/* Revision:    07 SEP 94   Version 1.0                                      */
/*                                                                           */
/* Function:                                                                 */
/*                                                                           */
/*   Procedure interfaces user programs with the "ntl" library utilities;    */
/* performs the check for overlapping intervals within the two given lists   */
/* of pairs of (absolute) positions; returns either a positive number, the   */
/* count of overlaps found, 0 if there were no overlaps, or negative number  */
/* indicating an error                                                       */
/*                                                                           */
/*****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include "ntl4.h"


/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* PROCEDURE: overlaps                                                       */
/*                                                                           */
/* Execution of unit code; returns the number of discovered overlaps         */

int overlaps (twoint_ptr list_1, twoint_ptr list_2)
{
 twoint_ptr scan_1, scan_2; long int low_1, high_1, low_2, high_2;
 int overlapped;

 overlapped = 0;
 for (scan_1 = list_1; scan_1 != NULL; scan_1 = scan_1 -> next) {
  for (scan_2 = list_2; scan_2 != NULL; scan_2 = scan_2 -> next) {
   if (scan_1 -> value_1 > scan_1 -> value_2) {
    low_1 = scan_1 -> value_2; high_1 = scan_1 -> value_1;
   }
   else { low_1 = scan_1 -> value_1; high_1 = scan_1 -> value_2; }
   if (scan_2 -> value_1 > scan_2 -> value_2) {
    low_2 = scan_2 -> value_2; high_2 = scan_2 -> value_1;
   }
   else { low_2 = scan_2 -> value_1; high_2 = scan_2 -> value_2; }
   if (NTL0_overlaps (low_1, high_1, low_2, high_2) == 1) overlapped++;
  }
 }
 return overlapped;
}

